//===============================================================
//
// File....... Demo_Pulse.c
// Compiler... CodeVisionAVR
// Revision... $Revision: 1.00 $
// Purpose.... Demo Code for US and IR Ranger
// Author..... Innovative Electronics, Inc.
// E-mail..... support@innovativeelectronics.com
// Started....
// Updated.... 29 FEB 2007
//==================================================================

// -----[ Program Description ]-------------------------------------
//
// This program demonstrates the use of US and IR Ranger 
// Module with PulseWidth interface and then send the measurement 
// results to UART Serial Port. 
//
// The distance measurement is done each second through an 
// ultrasonic ranger of the Smart Sense module. The output is four 
// ASCII character digits representing the range in millimeters (mm), 
// followed by carriage return (ASCII 13). The baudrate 9600, 8 bits, 
// no parity, with one stop bit.
//
// Smart Sense module address is set to 0xEE
// Chip type           : AT90S2313
// Clock frequency     : 4.000000 MHz
// SIG pin             : PORTB.2
// RDY/BUSY pin        : PINB.3

// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the AT90S2313


#pragma used+
sfrb ACSR=8;
sfrb UBRR=9;
sfrb UCR=0xa;
sfrb USR=0xb;
sfrb UDR=0xc;
sfrb PIND=0x10;
sfrb DDRD=0x11;
sfrb PORTD=0x12;
sfrb PINB=0x16;
sfrb DDRB=0x17;
sfrb PORTB=0x18;
sfrb EECR=0x1c;
sfrb EEDR=0x1d;
sfrb EEAR=0x1e;
sfrb WDTCR=0x21;
sfrb ICR1L=0x24;
sfrb ICR1H=0x25;
sfrw ICR1=0x24;   // 16 bit access
sfrb OCR1L=0x2a;
sfrb OCR1H=0x2b;
sfrw OCR1=0x2a;   // 16 bit access
sfrb TCNT1L=0x2c;
sfrb TCNT1H=0x2d;
sfrw TCNT1=0x2c;  // 16 bit access
sfrb TCCR1B=0x2e;
sfrb TCCR1A=0x2f;
sfrb TCNT0=0x32;
sfrb TCCR0=0x33;
sfrb MCUSR=0x34;
sfrb MCUCR=0x35;
sfrb TIFR=0x38;
sfrb TIMSK=0x39;
sfrb GIFR=0x3a;
sfrb GIMSK=0x3b;
sfrb SPL=0x3d;
sfrb SREG=0x3f;
#pragma used-


// Interrupt vectors definitions


// Needed by the power management functions (sleep.h)
#asm
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x20
	.EQU __sm_mask=0x10
	.SET power_ctrl_reg=mcucr
	#endif
#endasm


// CodeVisionAVR C Compiler
// (C) 1998-2006 Pavel Haiduc, HP InfoTech S.R.L.

// Prototypes for standard I/O functions


// CodeVisionAVR C Compiler
// (C) 1998-2002 Pavel Haiduc, HP InfoTech S.R.L.

// Variable length argument list macros



typedef char *va_list;





#pragma used+

char getchar(void);
void putchar(char c);
void puts(char *str);
void putsf(char flash *str);

char *gets(char *str,unsigned char len);

void printf(char flash *fmtstr,...);
void sprintf(char *str, char flash *fmtstr,...);
void snprintf(char *str, unsigned char size, char flash *fmtstr,...);
void vprintf (char flash * fmtstr, va_list argptr);
void vsprintf (char *str, char flash * fmtstr, va_list argptr);
void vsnprintf (char *str, unsigned char size, char flash * fmtstr, va_list argptr);
signed char scanf(char flash *fmtstr,...);
signed char sscanf(char *str, char flash *fmtstr,...);




                                   

            


#pragma used-

#pragma library stdio.lib


// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.


#pragma used+

void delay_us(unsigned int n);
void delay_ms(unsigned int n);

#pragma used-


typedef unsigned int   uint16_t;


uint16_t US;

void sendData(void)
{
    putchar('J');
    putchar('a');
    putchar('r');
    putchar('a');
    putchar('k');
    putchar(':');
    putchar(' ');
    putchar((US/1000)%10 + 0x30 );
    putchar((US/100)%10  + 0x30 );
    putchar((US%100)/10  + 0x30 );
    putchar((US%10)      + 0x30 );
    putchar(' ');
    putchar('m');
    putchar('m');

    putchar(0x0D);
}

void main(void)
{
unsigned int counter;
// Input/Output Ports initialization

// Port B initialization
PORTB=0x0C;
DDRB=0x04;

// UART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// UART Receiver: On
// UART Transmitter: On
// UART Baud rate: 9600
UCR=0x18;
UBRR=0x19;

delay_ms(1000);
while (1)
    {
        DDRB.2  = 1;    // set SIG pin as output

        PORTB.2  = 0;    //
        delay_us(50);    // send start pulse
        PORTB.2  = 1;    //

        DDRB.2  = 0;    // set SIG pin as input

        TCNT1=0; 
        counter=0;
        while (PINB.2 && counter<30000)     // wait for return pulse
        counter++;
        TCCR1B=0x02;        // start timer (timer period = 2uS)
        while ((!PINB.2) && !(TIFR&0x80));
        TCCR1B=0x00;        // stop timer
        US = TCNT1;         // save timer value to variable US

        US = US/5;          // convert to milimeters

        sendData();
        delay_ms(100);
    };
}

/*
US=0; //TCCR1B=0x02;        // start timer (timer period = 2uS)
        while ((!SIG_in) && US<40000)
        {
        delay_us(70);
        US++;  
        }//1 us =0.1mm
        //US=US;    
            // timing the return pulse
        //TCCR1B=0x00;        // stop timer
        //US = TCNT1;         // save timer value to variable US

        US = US*7;          // c */
